module TaggableController
  extend ActiveSupport::Concern

  included do
    before_action :authenticate_user!, only: [:tagging, :untagging]
    before_action :set_owner, only: [:tagging, :untagging]
  end

  # POST /v1/learning_objects/1/tagging
  # POST /v1/learning_objects/1/tagging.json
  def tagging
    @owner.tag(taggable, with: [tag_params[:name]])
    render json: taggable, status: :created
  end

  # DELETE /v1/learning_objects/1/untagging
  # DELETE /v1/learning_objects/1/untagging.json
  def untagging
    @owner.untag(taggable, tag_params[:name])
    render json: taggable, status: :ok
  end

  protected

  def taggable
    raise NotImplementedError
  end

  def tag_params
    params.require(:tag).permit(:name, :owner_id, :owner_type)
  end

  def set_owner
    if current_user.is_admin?
      @owner = tag_params[:owner_type].constantize.find(tag_params[:owner_id])
    else
      if tag_params[:owner_type] == 'Institution'
        if Institution.find(tag_params[:owner_id]).users.include? current_user
          @owner = Institution.find(tag_params[:owner_id])
        end
      else
        @owner = current_user
      end
    end
  end

end
