class V1::Users::BookmarksController < ApplicationController
  before_action :authenticate_user!
  before_action :set_user
  before_action :authorize!
  before_action :find_object, only: [:create, :destroy]

  # GET /v1/users/:user_id/bookmarks
  # GET /v1/users/:user_id/bookmarks.json
  def index
    render json: @user.bookmarks, status: :ok
  end

  # POST /v1/users/:user_id/bookmarks
  # POST /v1/users/:user_id/bookmarks.json
  def create
    bookmark = Bookmark.new(user: @user, bookmarkable: @object)
    if bookmark.save
      render json: bookmark, status: :created
    else
      render json: bookmark.errors, status: :unprocessable_entity
    end
  end

  def destroy
    Bookmark.where(user: @user, bookmarkable: @object).destroy_all
    render nothing: true, status: :ok
  end

  private

  def authorize!
    authorize Bookmark.new(user: @user)
  end

  def set_user
    @user = User.find(params[:id])
  end

  def find_object
    if %w(LearningObject Collection).include? bookmarks_params[:type]
      klass = bookmarks_params[:type].constantize
      @object = klass.find bookmarks_params[:id]
    else
      return render nothing: true, status: :not_acceptable
    end
  end

  def bookmarks_params
    params.require(:bookmarks).permit(:id, :type)
  end

end
