class ConvertVideoWorker
  include Sidekiq::Worker

  sidekiq_options queue: :convert_video

  def perform(id)
    begin
      object = LearningObject.find id

      dspace_client = DspaceService.create_client
      dspace_object = dspace_client.items.find(id: object.id_dspace, expand: 'bitstreams')

      # convert if object has a flv and not a mp4
      video = nil
      dspace_object.bit_streams.each do |bitstream|
        video = bitstream if bitstream.name =~ /(.flv|.wmv|.mov|.mpg)$/i

        return true if bitstream.name =~ /.mp4$/i
      end
      return true if video.nil?

      # retrieve video from dspace
      original_bitstream = dspace_client.bitstreams.retrieve(id: video.id)

      # open video with ffmpeg
      # FFMPEG.ffmpeg_binary = '/usr/local/bin/ffmpeg'
      FFMPEG.ffmpeg_binary = 'ffmpeg'
      file = ::FFMPEG::Movie.new(original_bitstream.path)
      return false unless file.valid? # if ffmpeg fails to read the movie

      converted_path = "#{original_bitstream.path}.mp4"

      # transcode (convert video to mp4)
      options = {
                  video_codec: "libx264",
                  x264_preset: "fast",
                  custom: '-movflags +faststart -pix_fmt yuv420p -vf "scale=trunc(iw/2)*2:trunc(ih/2)*2" -profile:v main -level 3.1 -refs 4 -b:a 160k -strict -2',
                  validate: true
                }
      file.transcode(converted_path, options)

      converted = File.open converted_path

      # send converted video to dspace
      dspace_client.items.add_bitstream(converted, id: object.id_dspace, name: "#{video.name.split('.')[0..-2].join('.')}.mp4", description: 'converted from original flv')

      p "Converted video for learning object #{id}"
    ensure
      # remove temp files
      original_bitstream.unlink unless original_bitstream.nil?
      converted.close unless converted.nil?
      FileUtils.rm(converted_path, force: true) unless converted_path.nil?
    end
  end

end
