# == Schema Information
#
# Table name: users
#
#  id                     :integer          not null, primary key
#  email                  :string           default(""), not null
#  encrypted_password     :string           default(""), not null
#  reset_password_token   :string
#  reset_password_sent_at :datetime
#  remember_created_at    :datetime
#  sign_in_count          :integer          default("0"), not null
#  current_sign_in_at     :datetime
#  last_sign_in_at        :datetime
#  current_sign_in_ip     :string
#  last_sign_in_ip        :string
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  provider               :string           default("email"), not null
#  uid                    :string           default(""), not null
#  confirmation_token     :string
#  confirmed_at           :datetime
#  confirmation_sent_at   :datetime
#  unconfirmed_email      :string
#  tokens                 :text
#  name                   :string
#  avatar_file_name       :string
#  avatar_content_type    :string
#  avatar_file_size       :integer
#  avatar_updated_at      :datetime
#  bookmarks_count        :integer          default("0")
#  user_category_id       :integer
#  score                  :float            default("0.0")
#  follows_count          :integer          default("0")
#

require 'test_helper'

class UserTest < ActiveSupport::TestCase
  should have_and_belong_to_many(:roles)
  should have_and_belong_to_many(:institutions)

  should have_many :bookmarks
  should have_many(:bookmark_collections).through(:bookmarks).source(:bookmarkable)
  should have_many(:bookmark_learning_objects).through(:bookmarks).source(:bookmarkable)
  should have_many :taggings
  should have_many :collections
  should have_many :learning_objects
  should have_many :views
  should have_many :downloads
  should have_many :likes
  should have_many :shares
  should have_many :follows
  should have_many :reviews

  test 'an user can follow another user' do
    john = users(:john)
    jack = users(:jack)
    assert john.follow(jack)

    # check jack followers
    assert_count 1, jack.followers
    assert_count 1, john.follows
    assert john.following? jack
  end

  test 'an user can follow a collection' do
    john = users(:john)
    ufpr_collection = collections(:ufpr)
    assert john.follow(ufpr_collection)

    # check ufpr followers
    assert_count 1, ufpr_collection.followers
    assert_count 1, john.follows
    assert john.following? ufpr_collection
  end

  test 'an user can unfollow a collection' do
    john = users(:john)
    ufpr_collection = collections(:ufpr)
    assert john.follow(ufpr_collection)

    # check ufpr followers
    assert_count 1, ufpr_collection.followers
    assert_count 1, john.follows
    assert john.following? ufpr_collection

    john.unfollow(ufpr_collection)

    # check ufpr followers
    assert_count 0, ufpr_collection.followers
    assert_count 0, john.follows
    assert_not john.following? ufpr_collection
  end
end
